﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;

namespace WawaSoft.CodeWawa {
    public class CodeProperty {
        public CodeProperty()
        {
        }
        public CodeProperty(string dbColName)
        {
            DbColName = dbColName;
        }

        private string dbColName;
        private string propertyName;
        private SqlDbType dbType;
        private Type csharpType;
        private string dbTypeStr;
        public string DbColName {
            get { return dbColName; }
            set {
                if (propertyName == dbColName)
                    propertyName = value;
                dbColName = value;
            }
        }
        public string DbTypeStr {
            get { return dbTypeStr; }
            set {
                dbTypeStr = value;
                switch (value.ToLower()) {
                    case "int":
                        csharpType = typeof(Int32?);
                        dbType = SqlDbType.Int;
                        break;
                    case "text":
                        csharpType = typeof(string);
                        dbType = SqlDbType.Text;
                        break;
                    case "bigint":
                        csharpType = typeof(Int64?);
                        dbType = SqlDbType.BigInt;
                        break;
                    case "binary":
                        csharpType = typeof(byte[]);
                        dbType = SqlDbType.Binary;
                        break;
                    case "bit":
                        csharpType = typeof(Boolean?);
                        dbType = SqlDbType.Bit;
                        break;
                    case "char":
                        csharpType = typeof(string);
                        dbType = SqlDbType.Char;
                        break;
                    case "datetime":
                        csharpType = typeof(DateTime?);
                        dbType = SqlDbType.DateTime;
                        break;
                    case "decimal":
                        csharpType = typeof(Decimal?);
                        dbType = SqlDbType.Decimal;
                        break;
                    case "float":
                        csharpType = typeof(Double?);
                        dbType = SqlDbType.Float;
                        break;
                    case "image":
                        csharpType = typeof(byte[]);
                        dbType = SqlDbType.Image;
                        break;
                    case "money":
                        csharpType = typeof(Decimal?);
                        dbType = SqlDbType.Money;
                        break;
                    case "nchar":
                        csharpType = typeof(string);
                        dbType = SqlDbType.NChar;
                        break;
                    case "ntext":
                        csharpType = typeof(string);
                        dbType = SqlDbType.NText;
                        break;
                    case "nvarchar":
                        csharpType = typeof(string);
                        dbType = SqlDbType.NVarChar;
                        break;
                    case "real":
                        csharpType = typeof(Single?);
                        dbType = SqlDbType.Real;
                        break;
                    case "smalldatetime":
                        csharpType = typeof(DateTime?);
                        dbType = SqlDbType.SmallDateTime;
                        break;
                    case "smallint":
                        csharpType = typeof(Int16?);
                        dbType = SqlDbType.SmallInt;
                        break;
                    case "smallmoney":
                        csharpType = typeof(Decimal?);
                        dbType = SqlDbType.SmallMoney;
                        break;
                    case "timestamp":
                        csharpType = typeof(DateTime?);
                        dbType = SqlDbType.Timestamp;
                        break;
                    case "tinyint":
                        csharpType = typeof(Byte?);
                        dbType = SqlDbType.TinyInt;
                        break;
                    case "uniqueidentifier":
                        csharpType = typeof(Guid?);
                        dbType = SqlDbType.UniqueIdentifier;
                        break;
                    case "varbinary":
                        csharpType = typeof(byte[]);
                        dbType = SqlDbType.VarBinary;
                        break;
                    case "varchar":
                        csharpType = typeof(string);
                        dbType = SqlDbType.VarChar;
                        break;
                    case "variant":
                        csharpType = typeof(Object);
                        dbType = SqlDbType.Variant;
                        break;
                    default:
                        throw new ApplicationException("unknow type" + value);
                }
            }
        }
        public SqlDbType DbType {
            get { return dbType; }
        }

        private int? dbLen;
        public int? DbLength
        {
            get
            {
                if(dbTypeStr != "varchar" 
                    && dbTypeStr != "nvarchar"
                    && dbTypeStr != "char"
                    && dbTypeStr != "nchar"
                    )
                    return null;
                return dbLen;
            }
            set
            {
                if (value == 2147483647) value = -1;
                dbLen = value;
            }
        }
        public bool DbAllowNull = true;
        public bool DbIsPk;
        public bool DbIsIdentity;

        public string PropertyName {
            get { return propertyName; }
            set { propertyName = value; }
        }
        public string Descript;
        public Type CSharpType
        {
            get { return csharpType; }
        }
        public string CSharpTypeStr
        {
            get
            {
                string temp = CSharpType.ToString().Replace("System.", "");
                if (temp.IndexOf("Nullable") != -1) {
                    temp = temp.Replace("`1", "");
                    temp = temp.Replace("[", "<");
                    temp = temp.Replace("]", ">");
                }
                return temp;
            }
        }

        public bool UiIsShowOn = true;
        public bool UiAllowEmpty = true;
        public long? UiMaxCheck;
        public long? UiMinCheck;
        public string UiRegxCheck;

        public override string ToString() {
            return dbColName;
        }
    }
    public class CodeModel {
        public CodeModel()
        {
        }

        public CodeModel(string tableName)
        {
            TableName = tableName;
        }

        private string tableName;
        private string className;
        public string TableName {
            get {
                return tableName;
            }
            set {
                if (className == tableName)
                    className = value;
                tableName = value;
            }
        }
        public string ClassName {
            get { return className; }
            set { className = value; }
        }

        public string PkColName;
        public string Namespace;
        public List<CodeProperty> Properties = new List<CodeProperty>();
        public override string ToString() {
            return tableName;
        }

        public string Descript;
    }
}
