﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;

namespace WawaSoft.CodeWawa {
    public partial class MainForm : Form {
       
        public MainForm() {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e) {
            string tmpls = System.Configuration.ConfigurationManager.AppSettings["tmpls"];
            string[] arr = tmpls.Split(';');
            foreach (string tmp in arr)
            {
                if(!string.IsNullOrEmpty(tmp))
                cmpTmpl.Items.Add(tmp);
            }
            if (cmpTmpl.Items.Count > 0) cmpTmpl.SelectedIndex = 0;
        }
        Dictionary<string,CodeModel> _models = new Dictionary<string, CodeModel>();
        private void toolStripButton8_Click(object sender, EventArgs e) {
            DataTable schemaTable = GetTables();
            lstModel.Items.Clear();
            lstProp.Items.Clear();
            clearModelProp();
            clearPropProp();
            foreach (DataRow row in schemaTable.Rows)
            {
                CodeModel model = new CodeModel(row["TABLE_NAME"].ToString());
                model.Namespace = txtDefaultNamespace.Text;
                _models[model.TableName] = model;
                lstModel.Items.Add(model);
            }
            foreach (KeyValuePair<string, CodeModel> pair in _models)
            {
                schemaTable = GetCols(pair.Key);
                foreach (DataRow row in schemaTable.Rows) {
                    CodeProperty property = new CodeProperty(row["ColumnName"].ToString());
                    property.DbTypeStr = row["DataTypeName"].ToString();
                    property.DbIsIdentity = Convert.ToBoolean(row["IsIdentity"]);
                    property.DbAllowNull = Convert.ToBoolean(row["AllowDBNull"]);
                    if (row["ColumnSize"] != DBNull.Value)
                        property.DbLength = Convert.ToInt32(row["ColumnSize"]);
                    if (row["IsKey"] != DBNull.Value)
                    {
                        property.DbIsPk = Convert.ToBoolean(row["IsKey"]);
                        if (property.DbIsPk) pair.Value.PkColName = property.DbColName;
                    }
                    pair.Value.Properties.Add(property);
                } 
            }
           
        }

        public DataTable GetTables() {
            SqlConnection conn = new SqlConnection(txtConnection.Text);
            conn.Open();
            DataTable schemaTable = conn.GetSchema(SqlClientMetaDataCollectionNames.Tables);
            conn.Close();
            return schemaTable;
        }
        public DataTable GetCols(string table) {
            SqlConnection conn = new SqlConnection(txtConnection.Text);
            SqlCommand cmd = new SqlCommand(string.Format("Select * from [{0}]",table), conn);
            conn.Open();

            SqlDataReader dataReader = cmd.ExecuteReader( CommandBehavior.KeyInfo | CommandBehavior.CloseConnection);
            DataTable schemaTable = dataReader.GetSchemaTable();
            dataReader.Close();
            return schemaTable;
        }

        private void lstModel_SelectedIndexChanged(object sender, EventArgs e) {
            CodeModel model = (CodeModel) lstModel.SelectedItem;
            if(model == null) return;
            lstProp.Items.Clear();
            clearModelProp();
            clearPropProp();
            foreach (CodeProperty property in model.Properties)
            {
                lstProp.Items.Add(property);
            }
            txtTableName.Text = model.TableName;
            txtClassName.Text = model.ClassName;
            txtNamespce.Text = model.Namespace;
            txtPKColName.Text = model.PkColName;
            txtModelDesc.Text = model.Descript;
        }
        void clearModelProp()
        {
            txtTableName.Text = "";
            txtClassName.Text = "";
            txtNamespce.Text = "";
            txtPKColName.Text = "";
            txtModelDesc.Text = "";
        }
        void clearPropProp()
        {
            txtDbColName.Text = "";
            cmbDbTypeStr.Text = "";
            txtDbLength.Text = "";
            chkDbAllowNull.Checked = false;
            chkDbIsIdentity.Checked = false;
            chkDbIsPk.Checked = false;
            txtCsharpType.Text = "";
            txtPropertyName.Text = "";
            txtPropDesc.Text = "";
        }

        private void lstProp_SelectedIndexChanged(object sender, EventArgs e) {
            CodeProperty property = (CodeProperty) lstProp.SelectedItem;
            if(property == null) return;
            clearPropProp();
            txtDbColName.Text = property.DbColName;
            cmbDbTypeStr.Text = property.DbTypeStr;
            txtDbLength.Text = property.DbLength.ToString();
            chkDbAllowNull.Checked = property.DbAllowNull;
            chkDbIsIdentity.Checked = property.DbIsIdentity;
            chkDbIsPk.Checked = property.DbIsPk;
            txtCsharpType.Text = property.CSharpTypeStr;
            txtPropertyName.Text = property.PropertyName;
            txtPropDesc.Text = property.Descript;
        }

        private void toolStripButton5_Click(object sender, EventArgs e) {
            if(lstModel.SelectedIndex == -1) return;
            txtCode.Text = CenSql((CodeModel)lstModel.SelectedItem);
        }

        private void toolStripButton6_Click(object sender, EventArgs e) {
            if (lstModel.SelectedIndex == -1) return;
            txtCode.Text = CenEntity((CodeModel)lstModel.SelectedItem);
        }
        private string CenSql(CodeModel model) {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("CREATE TABLE [dbo].[{0}](\r\n", model.TableName);
            List<CodeProperty> properties = model.Properties;
            int i = 0;
            foreach (CodeProperty property in properties) {
                string colName = property.DbColName;
                string type = property.DbTypeStr;
                string identity = property.DbIsIdentity ? "IDENTITY(1,1)" : "";
                string length = "";
                if(property.DbLength != null)
                {
                    if (property.DbLength.Value == -1)
                    {
                        length = "(max)";
                    }
                    else
                    {
                        length = string.Format("({0})", property.DbLength);
                    }
                }
                string allowNull = property.DbAllowNull ? "" : "NOT NULL";
                sb.AppendFormat("\t[{0}] [{1}] {2} {3} {4}", colName, type, identity, length, allowNull);
                if (i++ != properties.Count - 1) sb.Append(",\r\n"); else sb.Append("\r\n");
            }
            if (model.PkColName != null) {
                sb.AppendFormat("CONSTRAINT [PK_{0}] PRIMARY KEY CLUSTERED \r\n", model.TableName);
                sb.AppendFormat("(\r\n\t[{0}] ASC\r\n)WITH (IGNORE_DUP_KEY = OFF) ON [PRIMARY]", model.PkColName);
            }
            sb.Append(") ON [PRIMARY]\r\nGO");
            return sb.ToString();
        }
        private string CenEntity(CodeModel model) {
            if(string.IsNullOrEmpty(cmpTmpl.Text)) return "";
            StringBuilder sb = new StringBuilder();
            string tmpPath = Path.Combine(Application.StartupPath,
                string.Format("tmpls\\{0}.txt",cmpTmpl.Text));
            string strTmpl;
            using (StreamReader sr = new StreamReader(tmpPath, Encoding.UTF8)) {
                strTmpl = sr.ReadToEnd();
            }
            StringReader reader = new StringReader(strTmpl);
            while (true) {
                string line = reader.ReadLine();
                if (line == null) break;
                line = line.TrimEnd();
                if (line != "$foreach.prop$") {
                    line = line.Replace("$model.namespace$", model.Namespace);
                    line = line.Replace("$model.classname$", model.ClassName);
                    line = line.Replace("$model.tablename$", model.TableName);
                    line = line.Replace("$model.pkcolname$", model.PkColName);
                    sb.Append(line);
                    sb.AppendLine();
                    continue;
                }
                if (line == "$foreach.prop$") {
                    List<CodeProperty> props = model.Properties;
                    List<string> foreachLines = new List<string>();
                    while ((line = reader.ReadLine().TrimEnd()) != "$endforeach$") {
                        foreachLines.Add(line);
                    }
                    foreach (CodeProperty prop in props) {
                        foreach (string str in foreachLines) {
                            line = str.Replace("$prop.property$", prop.PropertyName);
                            line = line.Replace("$prop.dbcolname$", prop.DbColName);
                            line = line.Replace("$prop.csharptype$", prop.CSharpTypeStr);
                            line = line.Replace("$prop.property2$", prop.PropertyName.ToLower());
                            sb.Append(line);
                            sb.AppendLine();
                        }
                    }
                }
            }
            return sb.ToString();
        }

        private void toolStripButton7_Click(object sender, EventArgs e) {
            if(string.IsNullOrEmpty(txtCode.Text)) return;
            Clipboard.SetText(txtCode.Text);
        }
    }
}
