﻿using System;
using System.Collections;
using System.Data;
using System.Data.Common;
using System.Configuration;
using System.Text;

namespace WawaSoft.Common {
    public class DbHelper {
        private readonly static string _dbProviderName = ConfigurationManager.AppSettings["DbHelperProvider"];
        public readonly string _dbConnectionString = ConfigurationManager.AppSettings["DbHelperConnectionString"];
        private static Hashtable _parmCache = Hashtable.Synchronized(new Hashtable());

        public event Action<string> SqlTrace;

        #region contructs
        public DbHelper() {

        }
        public DbHelper(string connectionString) {
            _dbConnectionString = connectionString;
        }
        #endregion

        #region createconnection
        public DbConnection CreateConnection() {
            DbProviderFactory dbfactory = DbProviderFactories.GetFactory(DbHelper._dbProviderName);
            DbConnection dbconn = dbfactory.CreateConnection();
            dbconn.ConnectionString = _dbConnectionString;
            dbconn.Open();
            return dbconn;
        }
        public static DbConnection CreateConnection(string connectionString) {
            DbProviderFactory dbfactory = DbProviderFactories.GetFactory(DbHelper._dbProviderName);
            DbConnection dbconn = dbfactory.CreateConnection();
            dbconn.ConnectionString = connectionString;
            dbconn.Open();
            return dbconn;
        }
        #endregion

        #region getcommand
        public DbCommand GetStoredProcCommond(string storedProcedure) {
            DbCommand dbCommand = CreateConnection().CreateCommand();
            dbCommand.CommandText = storedProcedure;
            dbCommand.CommandType = CommandType.StoredProcedure;
            return dbCommand;
        }
        public DbCommand GetStoredProcCommond(string sp, DatabaseTrans trans) {
            DbCommand dbCommand = trans.DbConnection.CreateCommand();
            dbCommand.Transaction = trans.DbTrans;
            dbCommand.CommandText = sp;
            dbCommand.CommandType = CommandType.StoredProcedure;
            return dbCommand;
        }
        public DbCommand GetSqlStringCommond(string sqlQuery) {
            DbCommand dbCommand = CreateConnection().CreateCommand();
            dbCommand.CommandText = sqlQuery;
            dbCommand.CommandType = CommandType.Text;
            return dbCommand;
        }
        public DbCommand GetSqlStringCommond(string sqlQuery, DatabaseTrans trans) {
            DbCommand dbCommand = trans.DbConnection.CreateCommand();
            dbCommand.Transaction = trans.DbTrans;
            dbCommand.CommandText = sqlQuery;
            dbCommand.CommandType = CommandType.Text;
            return dbCommand;
        }
        #endregion

        #region 参数相关
        private void FillParameters(DbCommand command, string[] parameters, object[] values) {
            Action<string> temp = SqlTrace;
            if(temp != null)
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendFormat("commandTest:{0}\r\n", command.CommandText);
                if (values != null)
                {
                    foreach (object o in values)
                    {
                        sb.AppendFormat("{0},", o);
                    }
                }
                temp(sb.ToString());
            }
            if (values == null || values.Length == 0)
                return;
            if (initParametersFromCache(command)) {
                for (int i = 0; i < parameters.Length; i++) {
                    object value = values[i];
                    command.Parameters[i].Value = value;
                }
            }
            else {
                for (int i = 0; i < parameters.Length; i++) {
                    string parameter = parameters[i];
                    object value = values[i];
                    DbParameter pam = command.CreateParameter();
                    pam.ParameterName = parameter;
                    pam.Value = value;
                    command.Parameters.Add(pam);
                }
                CachedParameters(command);
            }

        }
        public void CachedParameters(DbCommand cmd) {
            DbParameterCollection paramColl = cmd.Parameters;
            DbParameter[] parms = new DbParameter[paramColl.Count];
            for (int i = 0; i < paramColl.Count; i++) {
                parms[i] = paramColl[i];
            }

            CacheParameters(
                string.Format("{0}{1}", cmd.Connection.ConnectionString, cmd.CommandText),
                parms);
        }
        public static void CacheParameters(string cacheKey, params DbParameter[] cmdParms) {
            _parmCache[cacheKey] = cmdParms;
        }
        public bool initParametersFromCache(DbCommand cmd) {
            DbParameter[] parms = GetCachedParameters(
                string.Format("{0}{1}", cmd.Connection.ConnectionString, cmd.CommandText));
            if (parms == null)
                return false;
            for (int i = 0; i < parms.Length; i++) {
                cmd.Parameters.Add(parms[i]);
            }
            return true;
        }
        public static DbParameter[] GetCachedParameters(string cacheKey) {
            DbParameter[] cachedParms = (DbParameter[])_parmCache[cacheKey];
            if (cachedParms == null)
                return null;

            DbParameter[] clonedParms = new DbParameter[cachedParms.Length];

            for (int i = 0, j = cachedParms.Length; i < j; i++)
                clonedParms[i] = (DbParameter)((ICloneable)cachedParms[i]).Clone();

            return clonedParms;
        }
        #endregion

        #region 内部执行方法
        private void CloseCmd(DbCommand cmd) {
            if (cmd != null && cmd.Connection.State != ConnectionState.Closed)
                cmd.Connection.Close();
        }

        private int ExecuteNonQuery(DbCommand cmd, string[] paramters, object[] values) {
            int ret;
            FillParameters(cmd, paramters, values);
            ret = cmd.ExecuteNonQuery();
            return ret;
        }
        private object ExecuteScalar(DbCommand cmd, string[] paramters,
            object[] values) {
            object ret;
            FillParameters(cmd, paramters, values);
            ret = cmd.ExecuteNonQuery();
            return ret;
        }
        private void ExecuteDateTable(DbCommand cmd, string[] paramters, object[] values, DataTable ret) {
            FillParameters(cmd, paramters, values);
            DbProviderFactory dbfactory = DbProviderFactories.GetFactory(_dbProviderName);
            DbDataAdapter dbDataAdapter = dbfactory.CreateDataAdapter();
            dbDataAdapter.SelectCommand = cmd;
            dbDataAdapter.Fill(ret);
        }
        private void ExecuteDateSet(DbCommand cmd, string[] paramters, object[] values, DataSet ret) {
            FillParameters(cmd, paramters, values);
            DbProviderFactory dbfactory = DbProviderFactories.GetFactory(_dbProviderName);
            DbDataAdapter dbDataAdapter = dbfactory.CreateDataAdapter();
            dbDataAdapter.SelectCommand = cmd;
            dbDataAdapter.Fill(ret);
        }
        private DbDataReader ExecuteReader(DbCommand cmd, string[] paramters, object[] values) {
            FillParameters(cmd, paramters, values);
            return cmd.ExecuteReader(CommandBehavior.CloseConnection);
        }
        #endregion

        #region 执行语句
        public int SqlExecuteNonQuery(string sql, string[] paramters, params object[] values) {
            int ret;
            DbCommand cmd = null;
            try {
                cmd = GetSqlStringCommond(sql);
                ret = ExecuteNonQuery(cmd, paramters, values);
            }
            finally {
                CloseCmd(cmd);
            }
            return ret;
        }
        public int SqlExecuteNonQuery(DatabaseTrans trans, string sql, string[] paramters, params object[] values) {
            int ret;
            DbCommand cmd = GetSqlStringCommond(sql, trans);
            ret = ExecuteNonQuery(cmd, paramters, values);
            return ret;
        }

        public object SqlExecuteScalar(string sql, string[] paramters, params object[] values) {
            object ret;
            DbCommand cmd = null;
            try {
                cmd = GetSqlStringCommond(sql);
                ret = ExecuteScalar(cmd, paramters, values);
            }
            finally {
                CloseCmd(cmd);
            }
            return ret;
        }
        public object SqlExecuteScalar(DatabaseTrans trans,
            string sql, string[] paramters, params object[] values) {
            object ret;
            DbCommand cmd = GetSqlStringCommond(sql, trans);
            ret = ExecuteScalar(cmd, paramters, values);
            return ret;
        }

        public DataTable SqlExecuteDateTable(string sql, string[] paramters, params object[] values) {
            DataTable ret = new DataTable();
            DbCommand cmd = null;
            try {
                cmd = GetSqlStringCommond(sql);
                ExecuteDateTable(cmd, paramters, values, ret);
            }
            finally {
                CloseCmd(cmd);
            }
            return ret;
        }

        public DataTable SqlExecuteDateTable(DatabaseTrans trans, string sql, string[] paramters,
            params object[] values) {
            DataTable ret = new DataTable();
            DbCommand cmd = GetSqlStringCommond(sql, trans);
            ExecuteDateTable(cmd, paramters, values, ret);
            return ret;
        }

        public DataSet SqlExecuteDateSet(string sql, string[] paramters, params object[] values) {
            DataSet ret = new DataSet();
            DbCommand cmd = null;
            try {
                cmd = GetSqlStringCommond(sql);
                ExecuteDateSet(cmd, paramters, values, ret);
            }
            finally {
                CloseCmd(cmd);
            }
            return ret;
        }
        public DataSet SqlExecuteDateSet(DatabaseTrans trans, string sql, string[] paramters,
            params object[] values) {
            DataSet ret = new DataSet();
            DbCommand cmd = GetSqlStringCommond(sql, trans);
            ExecuteDateSet(cmd, paramters, values, ret);
            return ret;
        }


        public DbDataReader SqlExecuteReader(string sql, string[] paramters, params object[] values) {
            DbCommand cmd = GetSqlStringCommond(sql);
            DbDataReader reader = ExecuteReader(cmd, paramters, values);
            return reader;
        }
        public DbDataReader SqlExecuteReader(DatabaseTrans trans, string sql, string[] paramters,
            params object[] values) {
            DbCommand cmd = GetSqlStringCommond(sql, trans);
            DbDataReader reader = ExecuteReader(cmd, paramters, values);
            return reader;
        }


        #endregion

        #region 执行存储过程
        public int SpExecuteNonQuery(string sql, string[] paramters, params object[] values) {
            int ret;
            DbCommand cmd = null;
            try {
                cmd = GetStoredProcCommond(sql);
                ret = ExecuteNonQuery(cmd, paramters, values);
            }
            finally {
                CloseCmd(cmd);
            }
            return ret;
        }
        public int SpExecuteNonQuery(DatabaseTrans trans, string sql, string[] paramters, params object[] values) {
            int ret;
            DbCommand cmd = GetStoredProcCommond(sql, trans);
            ret = ExecuteNonQuery(cmd, paramters, values);
            return ret;
        }

        public object SpExecuteScalar(string sql, string[] paramters, params object[] values) {
            object ret;
            DbCommand cmd = null;
            try {
                cmd = GetStoredProcCommond(sql);
                ret = ExecuteScalar(cmd, paramters, values);
            }
            finally {
                CloseCmd(cmd);
            }
            return ret;
        }
        public object SpExecuteScalar(DatabaseTrans trans,
            string sql, string[] paramters, params object[] values) {
            object ret;
            DbCommand cmd = GetStoredProcCommond(sql, trans);
            ret = ExecuteScalar(cmd, paramters, values);
            return ret;
        }

        public DataTable SpExecuteDateTable(string sql, string[] paramters, params object[] values) {
            DataTable ret = new DataTable();
            DbCommand cmd = null;
            try {
                cmd = GetStoredProcCommond(sql);
                ExecuteDateTable(cmd, paramters, values, ret);
            }
            finally {
                CloseCmd(cmd);
            }
            return ret;
        }

        public DataTable SpExecuteDateTable(DatabaseTrans trans, string sql, string[] paramters,
            params object[] values) {
            DataTable ret = new DataTable();
            DbCommand cmd = GetStoredProcCommond(sql, trans);
            ExecuteDateTable(cmd, paramters, values, ret);
            return ret;
        }

        public DataSet SpExecuteDateSet(string sql, string[] paramters, params object[] values) {
            DataSet ret = new DataSet();
            DbCommand cmd = null;
            try {
                cmd = GetStoredProcCommond(sql);
                ExecuteDateSet(cmd, paramters, values, ret);
            }
            finally {
                CloseCmd(cmd);
            }
            return ret;
        }
        public DataSet SpExecuteDateSet(DatabaseTrans trans, string sql, string[] paramters,
            params object[] values) {
            DataSet ret = new DataSet();
            DbCommand cmd = GetStoredProcCommond(sql, trans);
            ExecuteDateSet(cmd, paramters, values, ret);
            return ret;
        }

        public DbDataReader SpExecuteReader(string sql, string[] paramters, params object[] values) {
            DbCommand cmd = GetStoredProcCommond(sql);
            DbDataReader reader = ExecuteReader(cmd, paramters, values);
            return reader;
        }
        public DbDataReader SpExecuteReader(DatabaseTrans trans, string sql, string[] paramters,
            params object[] values) {
            DbCommand cmd = GetStoredProcCommond(sql, trans);
            DbDataReader reader = ExecuteReader(cmd, paramters, values);
            return reader;
        }
        #endregion
    }
}
