﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Text;

namespace WawaSoft.Common {
    public abstract class EntityBase {
        private static DbHelper _db = new DbHelper();
        protected string _tableName;
        protected string _PkName;
        protected abstract IList<string> Cols {
            get;
        }
        protected List<DbCommonClass<string, object>> _innerData = new List<DbCommonClass<string, object>>();

        #region public property
        public List<DbCommonClass<string, object>> InnerData {
            get { return _innerData; }
        }
        public string TableName {
            get { return _tableName; }
        }
        public string PKName {
            get { return _PkName; }
        }
        #endregion

        #region public methods
        public static void AddTracer(Action<string> tracer) {
            _db.SqlTrace += tracer;
        }
        public static void RemoveTracer(Action<string> tracer) {
            _db.SqlTrace -= tracer;
        }
        public static void Insert(EntityBase entity) {
            string sql = GetInsertSql(entity);
            string[] parameters = GetParameters(entity.InnerData);
            object[] parameterValues = GetParameterValuess(entity.InnerData);
            _db.SqlExecuteNonQuery(sql, parameters, parameterValues);
        }

        public static void Update(EntityBase entity) {
            string sql = GetUpdateSql(entity);
            string[] parameters = GetParameters(entity.InnerData);
            object[] parameterValues = GetParameterValuess(entity.InnerData);
            _db.SqlExecuteNonQuery(sql, parameters, parameterValues);
        }

        public static void Update(EntityBase entity, WhereCondition condition) {
            string sql = GetUpdateSql(entity, condition);
            string[] parameters = GetParameters(entity.InnerData, condition.InnerData);
            object[] parameterValues = GetParameterValuess(entity.InnerData, condition.InnerData);
            _db.SqlExecuteNonQuery(sql, parameters, parameterValues);
        }



        public static void Delete(EntityBase entity) {
            string sql = GetDeleteSql(entity);
            string[] parameters = GetParameters(entity.InnerData);
            object[] parameterValues = GetParameterValuess(entity.InnerData);
            _db.SqlExecuteNonQuery(sql, parameters, parameterValues);
        }
        public static void Delete(EntityBase entity, WhereCondition condition) {
            string sql = GetDeleteSql(entity, condition);
            string[] parameters = GetParameters(condition.InnerData);
            object[] parameterValues = GetParameterValuess(condition.InnerData);
            _db.SqlExecuteNonQuery(sql, parameters, parameterValues);
        }

        public static IList<T> Select<T>(string[] selectCols, WhereCondition condition)
            where T : EntityBase, new() {
            return Select<T>(-1,null,selectCols,condition);
        }
        public static IList<T> Select<T>(string orderByStr, string[] selectCols, WhereCondition condition)
    where T : EntityBase, new() {
            return Select<T>(-1, orderByStr, selectCols, condition);
        }
        public static IList<T> Select<T>(int topCount, string[] selectCols, WhereCondition condition)
where T : EntityBase, new() {
            return Select<T>(topCount, null, selectCols, condition);
        }
        public static IList<T> Select<T>(int topCount, string orderByStr, WhereCondition condition)
where T : EntityBase, new() {
            return Select<T>(topCount, orderByStr, new string[] { "*" }, condition);
        }
        public static IList<T> Select<T>(WhereCondition condition)
where T : EntityBase, new() {
            return Select<T>(-1, null, new string[]{"*"}, condition);
        }
        public static IList<T> Select<T>()
where T : EntityBase, new() {
            return Select<T>(-1, null, new string[] { "*" }, null);
        }
        public static IList<T> Select<T>(int topCount)
where T : EntityBase, new() {
            return Select<T>(topCount, null, new string[] { "*" }, null);
        }
        public static IList<T> Select<T>(int topCount,WhereCondition condition)
where T : EntityBase, new() {
            return Select<T>(topCount, null, new string[] { "*" }, condition);
        }
        public static IList<T> Select<T>(int topCount, string orderBystr)
where T : EntityBase, new() {
            return Select<T>(topCount, orderBystr, new string[] { "*" }, null);
        }
        public static IList<T> Select<T>(int topCount,string orderByStr, string[] selectCols, WhereCondition condition)
    where T : EntityBase, new() {
            string sql = GetSelectSql(topCount,orderByStr, new T().TableName, selectCols, condition);
            string[] parameters = null;
            object[] parameterValues = null;
            if (condition != null) {
                parameters = GetParameters(condition.InnerData);
                parameterValues = GetParameterValuess(condition.InnerData);
            }
            List<T> ret = new List<T>();
            using (IDataReader dr = _db.SqlExecuteReader(sql, parameters, parameterValues)) {
                while (dr.Read()) {
                    List<DbCommonClass<string, object>> list = new List<DbCommonClass<string, object>>();
                    for (int i = 0; i < dr.FieldCount; i++) {
                        list.Add(new DbCommonClass<string, object>(dr.GetName(i), dr[i]));
                    }
                    T t = new T();
                    t.ConvertToEntity(list);
                    ret.Add(t);
                }
            }
            return ret;
        }

        public void ClearInnerData() {
            _innerData.Clear();
        }
        #endregion

        #region protected methods

        public abstract void ConvertToEntity(IEnumerable<DbCommonClass<string, object>> items);

        protected void AddInnerData(string name, object value) {
            if (value != null)
                _innerData.Add(new DbCommonClass<string, object>(name, value));
            else {
                DbCommonClass<string, object> needRemove = null;
                foreach (DbCommonClass<string, object> commonClass in _innerData) {
                    if (commonClass.Key == name) {
                        needRemove = commonClass;
                        break;
                    }
                }
                if (needRemove != null) _innerData.Remove(needRemove);
            }
        }
        #endregion

        #region private methods
        private static string GetSelectSql(int topCount, string orderByStr, string tableName, string[] cols, WhereCondition condition) {
            int len = cols.Length;
            StringBuilder sql = new StringBuilder();
            sql.Append("SELECT\r\n");
            if (topCount != -1) sql.AppendFormat("top {0}\r\n", topCount);
            for (int i = 0; i < len; i++) {
                if (i != len - 1)
                    if (cols[i] == "*")
                        sql.AppendFormat("{0},", cols[i]);
                    else
                        sql.AppendFormat("[{0}],", cols[i]);
                else
                    if (cols[i] == "*")
                        sql.AppendFormat("{0}", cols[i]);
                    else
                        sql.AppendFormat("[{0}]", cols[i]);
            }
            sql.AppendFormat("\r\nFROM [{0}]\r\n", tableName);
            if (condition != null) sql.AppendFormat("WHERE {0}", condition);
            if (!string.IsNullOrEmpty(orderByStr))
                sql.AppendFormat("\r\nORDER BY {0}", orderByStr);
            return sql.ToString();
        }
        private static string GetSelectSql(string tableName, string[] cols, WhereCondition condition) {
            return GetSelectSql(-1, null, tableName, cols, condition);
        }
        private static string GetUpdateSql(EntityBase entity) {
            bool hasPkName = false;
            int len = entity.InnerData.Count;
            StringBuilder sql = new StringBuilder();
            sql.AppendFormat("UPDATE [{0}]\r\nSET\r\n", entity.TableName);
            for (int i = 0; i < len; i++) {
                DbCommonClass<string, object> obj = entity.InnerData[i];
                if (obj.Key == entity.PKName) {
                    hasPkName = true;
                    continue;
                }
                if (i != len - 1)
                    sql.AppendFormat("[{0}]=@{0},", entity.InnerData[i].Key);
                else
                    sql.AppendFormat("[{0}]=@{0}", entity.InnerData[i].Key);
            }
            if (!hasPkName) throw new ApplicationException("没有指定主键");
            sql.Append("\r\nWHERE\r\n");
            sql.AppendFormat("{0} = @{0}", entity.PKName);
            return sql.ToString();
        }
        private static string GetUpdateSql(EntityBase entity, WhereCondition condition) {
            int len = entity.InnerData.Count;
            StringBuilder sql = new StringBuilder();
            sql.AppendFormat("UPDATE [{0}]\r\nSET\r\n", entity.TableName);
            for (int i = 0; i < len; i++) {
                DbCommonClass<string, object> obj = entity.InnerData[i];
                if (i != len - 1)
                    sql.AppendFormat("[{0}]=@{0},", entity.InnerData[i].Key);
                else
                    sql.AppendFormat("[{0}]=@{0}", entity.InnerData[i].Key);
            }
            if (condition != null) sql.AppendFormat("\r\nWHERE {0}", condition);
            return sql.ToString();
        }
        private static string GetDeleteSql(EntityBase entity) {
            bool hasPkName = false;
            int len = entity.InnerData.Count;
            StringBuilder sql = new StringBuilder();
            sql.AppendFormat("DELETE FROM [{0}]\r\n", entity.TableName);
            for (int i = 0; i < len; i++) {
                DbCommonClass<string, object> obj = entity.InnerData[i];
                if (obj.Key == entity.PKName) {
                    hasPkName = true;
                    continue;
                }
            }
            if (!hasPkName) throw new ApplicationException("没有指定主键");
            sql.Append("\r\nWHERE\r\n");
            sql.AppendFormat("{0} = @{0}", entity.PKName);
            return sql.ToString();
        }
        private static string GetDeleteSql(EntityBase entity, WhereCondition condition) {
            StringBuilder sql = new StringBuilder();
            sql.AppendFormat("DELETE FROM [{0}]\r\n", entity.TableName);
            sql.AppendFormat("\r\nWHERE {0}", condition);
            return sql.ToString();
        }
        private static string GetInsertSql(EntityBase entity) {
            int len = entity.InnerData.Count;
            StringBuilder sql = new StringBuilder();
            sql.AppendFormat("INSERT INTO [{0}]\r\n", entity.TableName);
            sql.Append("(\r\n");
            for (int i = 0; i < len; i++) {
                if (i != len - 1)
                    sql.AppendFormat("[{0}],", entity.InnerData[i].Key);
                else
                    sql.AppendFormat("[{0}]", entity.InnerData[i].Key);
            }
            sql.Append(")\r\n");
            sql.Append("VALUES(\r\n");
            for (int i = 0; i < len; i++) {
                if (i != len - 1)
                    sql.AppendFormat("@{0},", entity.InnerData[i].Key);
                else
                    sql.AppendFormat("@{0}", entity.InnerData[i].Key);
            }
            sql.Append(")\r\n");
            return sql.ToString();
        }
        private static string[] GetParameters(IList<DbCommonClass<string, object>> items) {
            int len = items.Count;
            List<string> parameters = new List<string>();
            for (int i = 0; i < len; i++) {
                parameters.Add(string.Format("@{0}", items[i].Key));
            }
            return parameters.ToArray();
        }
        private static object[] GetParameterValuess(List<DbCommonClass<string, object>> items) {
            int len = items.Count;
            List<object> parameters = new List<object>();
            for (int i = 0; i < len; i++) {
                parameters.Add(items[i].Value);
            }
            return parameters.ToArray();
        }
        private static object[] GetParameterValuess(List<DbCommonClass<string, object>> list, List<DbCommonClass<string, object>> list2) {
            int len = list.Count;
            List<object> parameters = new List<object>();
            for (int i = 0; i < len; i++) {
                parameters.Add(list[i].Value);
            }
            len = list2.Count;
            for (int i = 0; i < len; i++) {
                parameters.Add(list2[i].Value);
            }
            return parameters.ToArray();
        }

        private static string[] GetParameters(List<DbCommonClass<string, object>> list, List<DbCommonClass<string, object>> list2) {
            int len = list.Count;
            List<string> parameters = new List<string>();
            for (int i = 0; i < len; i++) {
                parameters.Add(string.Format("@{0}", list[i].Key));
            }
            len = list2.Count;
            for (int i = 0; i < len; i++) {
                string param = string.Format("@{0}", list2[i].Key);
                if (parameters.Contains(param))
                    throw new ApplicationException("参数名重复");
                parameters.Add(param);
            }
            return parameters.ToArray();
        }
        #endregion
    }
}
