using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using WawaSoft.Common;

namespace WawaSoft.Common {
    public class PageingHelper {
        static readonly DbHelper _db;
        static PageingHelper() {
            try {
                _db = new DbHelper();
            }
            catch (Exception ex) {
                System.Diagnostics.Trace.TraceError(ex.ToString());
                throw;
            }
        }

        #region WawaPageing

        public static List<Dictionary<string, object>> WawaPageing(int page, int pageSize,
            string tableName, string fields, string strOrderBy, string strWhere, out short maxPage,
            out int totalRow, out string Descript) {

            List<Dictionary<string, object>> ret = new List<Dictionary<string, object>>();
            DbCommand command = _db.GetStoredProcCommond("Pagination");

            if (_db.initParametersFromCache(command)) {
                command.Parameters["@Page"].Value = page;
                command.Parameters["@PageSize"].Value = pageSize;
                command.Parameters["@Field"].Value = fields;
                command.Parameters["@OrderBy"].Value = strOrderBy;
                command.Parameters["@Filter"].Value = strWhere;
            }
            else {
                DbParameter p = command.CreateParameter();
                p.ParameterName = "@Page";
                p.Direction = ParameterDirection.Input;
                p.Value = page;
                command.Parameters.Add(p);

                p = command.CreateParameter();
                p.ParameterName = "@PageSize";
                p.Direction = ParameterDirection.Input;
                p.Value = pageSize;
                command.Parameters.Add(p);

                p = command.CreateParameter();
                p.ParameterName = "@Table";
                p.Direction = ParameterDirection.Input;
                p.Value = tableName;
                command.Parameters.Add(p);

                p = command.CreateParameter();
                p.ParameterName = "@Field";
                p.Direction = ParameterDirection.Input;
                p.Value = fields;
                command.Parameters.Add(p);

                p = command.CreateParameter();
                p.ParameterName = "@OrderBy";
                p.Direction = ParameterDirection.Input;
                p.Value = strOrderBy;
                command.Parameters.Add(p);

                p = command.CreateParameter();
                p.ParameterName = "@Filter";
                p.Direction = ParameterDirection.Input;
                p.Value = strWhere;
                command.Parameters.Add(p);

                p = command.CreateParameter();
                p.ParameterName = "@MaxPage";
                p.Size = 4;
                p.Direction = ParameterDirection.Output;
                command.Parameters.Add(p);

                p = command.CreateParameter();
                p.ParameterName = "@TotalRow";
                p.Size = 8;
                p.Direction = ParameterDirection.Output;
                command.Parameters.Add(p);

                p = command.CreateParameter();
                p.ParameterName = "@Descript";
                p.Size = 100;
                p.Direction = ParameterDirection.Output;
                command.Parameters.Add(p);

                _db.CachedParameters(command);
            }


            using (IDataReader dr = command.ExecuteReader()) {
               Dictionary<string, object> record;
                while (dr.Read()) {
                    record = new Dictionary<string, object>();
                    for (int i = 0; i < dr.FieldCount; i++) {
                        record.Add(dr.GetName(i), dr[i]);
                    }
                    ret.Add(record);
                }
            }
            Descript = (string)command.Parameters["@Descript"].Value;
            if (Descript != "successful") throw new ApplicationException(Descript);
            maxPage = short.Parse(command.Parameters["@MaxPage"].Value.ToString());
            totalRow = int.Parse(command.Parameters["@TotalRow"].Value.ToString());
            
            return ret;

        }
        #endregion
    }
}
