using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using WawaSoft.Common;

namespace WawaSoft.Common
{
    public class SensitiveFilter
    {
        static readonly DbHelper _db;
        static readonly List<string> _sensits = new List<string>();
        private static DateTime _lastRefreshTime = DateTime.Now;
        static object _syncRoot = new object();
        static SensitiveFilter()
        {
            try
            {
                _db = new DbHelper();
                loadSensits();
            }
            catch(Exception ex)
            {
                System.Diagnostics.Trace.TraceError(ex.ToString());
                throw;
            }
        }

        private static void loadSensits()
        {
            string sql = "SELECT [Words] FROM [G_SensitiveWords]";
            using (IDataReader dr = _db.SqlExecuteReader(sql,null))
            {
                _sensits.Clear();
                while (dr.Read())
                {
                    _sensits.Add(dr.GetString(0));
                }
                _lastRefreshTime = DateTime.Now;
            }
        }

        public static bool ContainSensitiveWords(string input, out string sensitWord)
        {
            lock (_syncRoot)
            {
                if (DateTime.Compare(_lastRefreshTime.AddMinutes(5), DateTime.Now) < 0)
                    loadSensits();
            }

            sensitWord = string.Empty;
            List<string> sensitWords = _sensits;

            foreach (string word in sensitWords)
            {
                if(input.Contains(word))
                {
                    sensitWord = word;
                    return true;
                }
            }
            return false;
        }
    }
}
