﻿using System;
using System.Collections.Generic;
using System.Text;

namespace WawaSoft.Common {
    public class WhereCondition {
        private StringBuilder innerStr = new StringBuilder();
        private Dictionary<string, object> innerData = new Dictionary<string, object>();
        public WhereCondition(string leftOperand, SqlOperator op, object Obj) {
            innerStr.AppendFormat(" {0} {1} @{0} ", leftOperand, getOpStr(op));
            innerData.Add(leftOperand, Obj);
        }
        public WhereCondition(string colName, string paramName, SqlOperator op, object Obj) {
            innerStr.AppendFormat(" {0} {1} @{2} ", colName, getOpStr(op), paramName);
            innerData.Add(paramName, Obj);
        }
        public WhereCondition(string condition)
        {
            innerStr.AppendFormat(" {0} ", condition);
        }

        private WhereCondition() {

        }

        private string getOpStr(SqlOperator op) {
            switch (op) {
                case SqlOperator.Equal:
                    return "=";
                case SqlOperator.NotEqual:
                    return "<>";
                case SqlOperator.GreaterThan:
                    return ">";
                case SqlOperator.LessThan:
                    return "<";
                case SqlOperator.Like:
                    return "like";
                default:
                    throw new ApplicationException("未知操作符");
            }
        }

        public List<DbCommonClass<string, object>> InnerData {
            get {
                List<DbCommonClass<string, object>> ret = new List<DbCommonClass<string, object>>();
                foreach (KeyValuePair<string, object> pair in innerData) {
                    ret.Add(new DbCommonClass<string, object>(pair.Key, pair.Value));
                }
                return ret;
            }
        }

        public WhereCondition And(WhereCondition condition) {
            WhereCondition ret = new WhereCondition();
            ret.innerStr.Append(innerStr.ToString());
            ret.mergerInnerData(innerData);
            ret.innerStr.Append(" and ");
            ret.innerStr.Append(condition.innerStr.ToString());
            ret.mergerInnerData(condition.innerData);
            return ret;
        }
        public WhereCondition Or(WhereCondition condition) {
            WhereCondition ret = new WhereCondition();
            ret.innerStr.Append(innerStr.ToString());
            ret.mergerInnerData(innerData);
            ret.innerStr.Append(" or ");
            ret.innerStr.Append(condition.innerStr.ToString());
            ret.mergerInnerData(condition.innerData);
            return ret;
        }
        public WhereCondition Group() {
            WhereCondition ret = new WhereCondition();
            ret.innerStr.AppendFormat("({0})", innerStr.ToString());
            ret.mergerInnerData(innerData);
            return ret;
        }
        public override string ToString() {
            return innerStr.ToString();
        }
        void mergerInnerData(IDictionary<string, object> dict) {
            foreach (KeyValuePair<string, object> pair in dict) {
                if (innerData.ContainsKey(pair.Key))
                    throw new ApplicationException("参数名重复" + pair.Key);
                innerData.Add(pair.Key, pair.Value);
            }
        }
    }

    public enum SqlOperator {
        Equal,
        NotEqual,
        GreaterThan,
        LessThan,
        Like,
    }
}
